#ifndef COMPARE_H
#define COMPARE_H
/**
 * @file compare.h
 * @author Keith Holman
 * @date Created: Friday, October 24, 2008
 *
 * @brief Defines the logic for comparing doubles and floats
 */

#include <float.h>
#include <math.h>

/**
 * @brief Safe double equality comparison
 * @param n1    first number to compare
 * @param n2    second number to compare
 * @retval  true    values are equal
 * @retval  false   values are not equal
 */
inline bool is_equal( double n1, double n2 )
{
  return ( fabs( fabs( n1 ) - fabs( n2 ) ) < DBL_EPSILON );
}

#if 0
/**
 * @brief Safe float equality comparison
 * @param n1    first number to compare
 * @param n2    second number to compare
 * @retval  true    values are equal
 * @retval  false   values are not equal
 */
inline bool is_equal( float n1, float n2 )
{
  return ( static_cast< float >( fabs( fabs( n1 ) - fabs( n2 ) ) < FLT_EPSILON ) );
}

/**
 * @brief Template Version of a safe equality comparison
 * @param n1    first number to compare
 * @param n2    second number to compare
 * @retval  true    values are equal
 * @retval  false   values are not equal
 */
template< typename T >
bool is_equal( T n1, T n2 )
{
  return n1 == n2;
}
#endif

/**
 * @brief Template Version of a safe inequality comparison
 * @param n1    first number to compare
 * @param n2    second number to compare
 * @retval  true    values are not equal
 * @retval  false   values are equal
 */
template< typename T >
bool is_not_equal( const T& n1, const T& n2 )
{
  return !is_equal( n1, n2 );
}

#endif // ifdef COMPARE_H
