/**
 * @file engager.cpp
 * @author Keith Holman
 * @date Created: Tuesday, October 14, 2008
 *
 * @brief Defines the atomic model for the chalk engager
 */

#include "engager.h"
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )
#include <cstdlib>

Engager::Engager( const std::string& name )
: Atomic( name ),
  _in( addInputPort( "in" ) ),
  _out( addOutputPort( "out" ) ),
  _chalkEngaged( false ),
  _preparationTime( 0, 0, 10, 0 )
{
  _chalkEngaged = atoi( MainSimulator::Instance().getParameter( description(), "engaged" ).c_str() );
	string time( MainSimulator::Instance().getParameter( description(), "preparation" ) ) ;

	if( time != "" )
		_preparationTime = time;
}

Model &Engager::initFunction()
{
	this->passivate();
	return *this;
}

Model &Engager::externalFunction( const ExternalMessage &msg )
{
	if( msg.port() == _in )
	{
    bool new_value = static_cast < bool >( msg.value() );
    if ( _chalkEngaged != new_value )
    {
      _chalkEngaged = new_value;
		  holdIn( active, _preparationTime );
    }
	}
	return *this;
}

Model &Engager::internalFunction( const InternalMessage & )
{
	passivate();
	return *this;
}

Model &Engager::outputFunction( const InternalMessage &msg )
{
  sendOutput( msg.time(), _out, _chalkEngaged );
	return *this ;
}

std::string Engager::className() const
{
	return "Engager";
}
