/**
 * @file queue.cpp
 * @author Keith Holman
 * @date Created: Tuesday, October 14, 2008
 *
 * @brief Defines the atomic model for the queue
 */

#include "queue.h"
#include "message.h"    // class ExternalMessage, InternalMessage
#include "mainsimu.h"   // MainSimulator::Instance().getParameter( ... )

Queue::Queue( const std::string& name )
: Atomic( name ),
  _in( addInputPort( "in" ) ),
  _ready( addInputPort( "ready" ) ),
  _out( addOutputPort( "out" ) ),
  _waiting( false )
{
}

Model &Queue::initFunction()
{
	this->passivate();
	return *this;
}

Model &Queue::externalFunction( const ExternalMessage &msg )
{
  if( msg.port() == _in )
  {
    _queue.push( msg.value() );
    holdIn( active, Time( 0, 0, 0, 0 ) );
  }
  else if ( msg.port() == _ready )
  {
    _waiting = false;
    _queue.pop();
    if ( !_queue.empty() )
    {
      holdIn( active, Time( 0, 0, 0, 0 ) );
    }
    else
    {
      passivate();
    }
  }
	return *this;
}

Model &Queue::internalFunction( const InternalMessage & )
{
	passivate();
	return *this;
}

Model &Queue::outputFunction( const InternalMessage &msg )
{
  if ( !_queue.empty() && !_waiting )
  {
    Value val = _queue.front();
    _waiting = true;
    sendOutput( msg.time(), _out, val );
  }
	return *this;
}

std::string Queue::className() const
{
	return "Queue";
}
